@@ -358,7 +358,6 @@ namespace acommon {
 
   static void get_lang_env(String & str) 
   {
-    // NOTE: THIS IS NOT THREAD SAFE
     if (proc_locale_str(getenv("LC_MESSAGES"), str)) return;
     if (proc_locale_str(getenv("LANG"), str)) return;
     if (proc_locale_str(getenv("LANGUAGE"), str)) return;
@@ -369,7 +368,7 @@ namespace acommon {
 
   static void get_lang(String & final_str) 
   {
-    // NOTE: THIS IS NOT THREAD SAFE
+    // FIXME: THIS IS NOT THREAD SAFE
     String locale = setlocale (LC_ALL, NULL);
     if (locale == "C")
       setlocale (LC_ALL, "");
@@ -1063,9 +1062,9 @@ namespace acommon {
     , {"filter",   KeyInfoList  , "url",
        N_("add or removes a filter")}
     , {"filter-path", KeyInfoList, FILTER_DIR,
-       N_("path(es) aspell looks for filters (.so/.dll/.flt);")}
+       N_("path(es) aspell looks for filters (.so/.dll/.flt)")}
     , {"option-path", KeyInfoList, FILTER_OPT_DIR,
-       N_("path(es) aspell looks for options descriptions;")}
+       N_("path(es) aspell looks for options descriptions")}
     , {"mode",     KeyInfoString, "url",             mode_string }
     , {"extra-dicts", KeyInfoList, "",
        N_("extra dictionaries to use")}
@@ -1136,12 +1135,16 @@ namespace acommon {
     //
     , {"backup",  KeyInfoBool, "true",
        N_("create a backup file by appending \".bak\"")}
-    , {"reverse", KeyInfoBool, "false",
-       N_("reverse the order of the suggest list")}
-    , {"time"   , KeyInfoBool, "false",
-       N_("time load time and suggest time in pipe mode")}
+    , {"include-guesses", KeyInfoBool, "false",
+       N_("Make possible root/affix combinations not in the dictionary")}
     , {"keymapping", KeyInfoString, "aspell",
        N_("keymapping for check mode, one of aspell ispell")}
+    , {"reverse", KeyInfoBool, "false",
+       N_("reverse the order of the suggest list")}
+    , {"suggest", KeyInfoBool, "true",
+       N_("suggest possible replacements")}
+    , {"time"   , KeyInfoBool, "false",
+       N_("time load time and suggest time in pipe mode")}
     
   };
 
