@@ -23,10 +23,13 @@ namespace acommon {
   {
   private:
     FILE * file_;
+    bool   own_;
 
   public:
-    FStream(char d = '\n') : IStream(d), file_(0) {}
-    FStream(FILE * f) : IStream('\n'), file_(f) {}
+    FStream(char d = '\n') 
+      : IStream(d), file_(0), own_(true) {}
+    FStream(FILE * f, bool own = true) 
+      : IStream('\n'), file_(f), own_(own) {}
     ~FStream() {close();}
 
     PosibErr<void> open(ParmString, const char *);
@@ -40,7 +43,7 @@ namespace acommon {
 
     // NOTE: Use c_stream only as a last resort as it may
     //       disappear if the underlining impl changes
-    std::FILE * c_stream();
+    FILE * c_stream();
     // However, file_no will always be available.
     int file_no();
 
@@ -61,10 +64,10 @@ namespace acommon {
     bool read(void *, unsigned int i);
     void write(ParmString);
     void write(char c);
-    void write(const char *, unsigned int i);
+    void write(const void *, unsigned int i);
 
     long int tell() {return ftell(file_);}
-    bool seek(long int offset, int whence) {
+    bool seek(long int offset, int whence = SEEK_SET) {
       return fseek(file_, offset, whence) == 0;
     }
     
