@@ -4,12 +4,15 @@
 // license along with this library if you did not you can find
 // it at http://www.gnu.org/.
 
+#define COMPILE_IN_FILTER // url filter is always static
+
 #include "indiv_filter.hpp"
+#include "key_info.hpp"
+#include "loadable-filter-API.hpp"
 
 namespace acommon {
 
-  class UrlFilter : public IndividualFilter 
-  {
+  class UrlFilter : public IndividualFilter {
   public:
     PosibErr<bool> setup(Config *);
     void reset() {}
@@ -18,7 +21,7 @@ namespace acommon {
 
   PosibErr<bool> UrlFilter::setup(Config *) 
   {
-    name_ = "url";
+    name_ = "url-filter";
     order_num_ = 0.95;
     return true;
   }
@@ -27,7 +30,7 @@ namespace acommon {
   {
     return c != '"' && c != ' ' && c != '\n' && c != '\t';
   }
-
+ 
   void UrlFilter::process(FilterChar * & str, FilterChar * & end)
   {
     for (FilterChar * cur = str; cur < end; ++cur) 
@@ -56,8 +59,5 @@ namespace acommon {
     }
   }
 
-  IndividualFilter * new_url_filter() 
-  {
-    return new UrlFilter();
-  }
+  ACTIVATE_FILTER(acommon,UrlFilter,url)
 }
