@@ -315,7 +315,7 @@ namespace aspeller_default_suggest {
       delete[] new_word;
     }
     const char * original_soundslike = original_word.soundslike.c_str();
-    unsigned int original_soundslike_len = strlen(original_soundslike);
+    //unsigned int original_soundslike_len = strlen(original_soundslike);
       
     EditDist (* edit_dist_fun)(const char *, const char *, 
 			       const EditDistanceWeights &);
@@ -342,6 +342,7 @@ namespace aspeller_default_suggest {
 	SoundslikeWord sw;	
 	EditDist score;
         unsigned int stopped_at = LARGE_NUM;
+        //CERR << "\\" << original_soundslike << '\n';
 	while ( (sw = els->next(stopped_at)) == true) 
 	{
 	  score = edit_dist_fun(sw.soundslike,
@@ -349,6 +350,7 @@ namespace aspeller_default_suggest {
 				parms.edit_distance_weights);
 	  stopped_at = score.stopped_at - sw.soundslike;
 	  if (score < LARGE_NUM) {
+            //CERR << "//" << sw.soundslike << ' ' << score << '\n';
 	    stopped_at = LARGE_NUM;
 	    BasicWordSet::Emul e = data_set->words_w_soundslike(sw);
 	    BasicWordInfo bw;
